function varargout=fxLoadEMGdata(input_filename)

% Loads Labview binary experiment files in to matlab space
% Examples of use
%
% No arguments-   load_data(file)    
% Each variable will be assigned into base workspace
%
% 2 Output arguments
% [Channels,Data]=load_data(file)
% Channels- Cell array of channel names
% Data- Matrix of channel data arranged by columns with 1st column being a
% time vector.
%
% 3 Output arguments
% [Channels,Data,ScanRate]=load_data(file)
% ScanRate - sampling frequency
%
% Written by Nicholas Au-Yong 2012

fid=fopen(input_filename,'r','ieee-be');
if fid == -1
   fprintf('\nERROR: file %s not found\n',input_filename);
   return;
end

%Header = initialize_Header_struct;

Header.header_length = fread(fid,1,'int32');
Header.Data_start_position=Header.header_length+4;  % 4 bytes offset for initial header length.
channel_string_length = fread(fid,1,'int32');
channel_string = char(fread(fid,channel_string_length,'char')');
Header.channel_number_cell = tokenize(channel_string,{','},{});
Header.num_channels = size(Header.channel_number_cell,2); 
channel_settings_length = fread(fid,1,'int32'); % group channel settings total length
group_array_length = fread(fid,1,'int32'); % # of elements in the group channel settings array

for i=1:group_array_length
    channel_string_length = fread(fid,1,'int32'); % length of the channel string in the group settings element
    group_settings.channel_name = char(fread(fid,channel_string_length,'char')');
    group_settings.upper_input_limits = fread(fid,1,'float32');
    group_settings.lower_input_limits = fread(fid,1,'float32');
    group_settings.range = fread(fid,1,'float32');
    group_settings.polarity = fread(fid,1,'uint16');
    group_settings.gain = fread(fid,1,'float32');
    group_settings.coupling = fread(fid,1,'uint16');
    group_settings.input_mode = fread(fid,1,'uint16');
    group_settings.scale_multiplier = fread(fid,1,'float32');
    group_settings.scale_offset = fread(fid,1,'float32');
    Header.group_settings(i) = group_settings;
end

Header.scan_rate = fread(fid,1,'float32');
Header.interchannel_delay = fread(fid,1,'float32');
channel_name_length = fread(fid,1,'int32');
channel_names = char(fread(fid,channel_name_length,'char')');
Header.channel_names = [{'Time'} tokenize(channel_names,{','},{})];
subject_info_length = fread(fid,1,'int32');
Header.subject_info = char(fread(fid,subject_info_length,'char')');
Header.data_position = ftell(fid);


status=fseek(fid,Header.Data_start_position,'bof');
data_array = fread(fid,[Header.num_channels+1,inf],'short')'; % read all the samples with +1 to consider time code a channel

for channel=1:Header.num_channels
    if Header.group_settings(channel).scale_multiplier ~= 1
       
        data_array(:,channel) = data_array(:,channel)*Header.group_settings(channel).scale_multiplier;
    end
    if Header.group_settings(channel).scale_offset ~= 0
        data_array(:,channel) = data_array(:,channel) + Header.group_settings(channel).scale_offset;
    end
end


%% Delete time code channel and add time array to first column

data_array=data_array(:,1:size(data_array,2)-1); %Delete time code channel
time_vector=[0:length(data_array)-1]/Header.scan_rate; % Create time vector
data_array=[time_vector' data_array]; % Add time vector to first column




%% Assigns variable to workspace

switch nargout
    case 0
        for I=1:length(Header.channel_names)
            assignin('caller',strrep(Header.channel_names{I},' ','_'),data_array(:,I));
            Channel_Names{I}=strrep(Header.channel_names{I},' ','_');
        end
        assignin('caller','Channel_Names',Channel_Names);
        assignin('caller','samprate',Header.scan_rate);
        
    case 2
        varargout{1}=Header.channel_names;
        varargout{2}=data_array;
        
    case 3
        varargout{1}=Header.channel_names;
        varargout{2}=data_array;
        varargout{3}=Header.scan_rate;
    case 4
        varargout{1}=Header.channel_names;
        varargout{2}=data_array;
        varargout{3}=Header.scan_rate;
        varargout{4}=Header.group_settings;
    otherwise
            disp('Invalid number of output arguments.')
        
end
        








function arg_cell = tokenize(input_string,delimiters_cell,reserved_characters)
% tokenize: convert a string to a cell array based on delimiters
% USAGE:
%   arg_cell = tokenize(input_string,delimiters_cell,reserved_characters)
% INPUTS:
%   input_string: string to tokenize
%   delimiters_cell: cell array with delimiters to use to divide string
%   reserved_characters: reserved characters to be removed from the arguments after they are tokenized

arg_cell = {};
delimiter_string = '';
delimiters_cell = delimiters_cell(:)';
for delim=1:size(delimiters_cell,2)
    delimiter_string = [delimiter_string sprintf(delimiters_cell{1,delim})];
end

while(any(input_string))
    [new_arg input_string] = strtok(input_string,delimiter_string);
    new_arg = remove_chars(new_arg,reserved_characters,2);
    if ~isempty(new_arg)
        arg_cell{1,(size(arg_cell,2)+1)} = new_arg;
    end
end


function newstring = remove_chars(string_in,unwanted_char_cell,preserve_words)
% remove_chars: remove characters from a short string, potentially preserving words
% USAGE:
%   remove_chars(string_in,unwanted_char_cell,preserve_words)
% INPUTS:
%   string_in: string to be processed
%   unwanted_char_cell: characters to be excised
%   preserve_words: 0 not to preserve words. To preserve words: 1 to replace or more unwanted chars with spaces 
%                   2 to preserve words with underscores
% OUTPUT:
%   newstring: a string with the characters removed
%

string_length = length(string_in);
newword = [];
newstring = [];

i=1;
while i<=string_length
    c=string_in(i);
    if preserve_words == 0
        switch c
        case unwanted_char_cell
        otherwise
            newstring = [newstring c]; % slow but simple
        end
    else
        switch c
        case unwanted_char_cell
            if ~isempty(newword)
                if preserve_words == 2
                    newstring = [newstring '_' newword];
                else
                    newstring = [newstring ' ' newword];
                end
                newword = [];
            end
        otherwise
            newword = [newword c]; % slow but simple
        end
    end
    i=i+1;
end
if ~isempty(newword)
    if preserve_words == 2
        newstring = [newstring '_' newword];
    else
        newstring = [newstring ' ' newword];
    end
end
if preserve_words > 0
    newstring = newstring(2:length(newstring));
end