function plotMEPAnimals(gnum, anode, cathode, current, varargin)
% This function plots the MEP for a given configuration across the weeks.
% USAGE
% PLOTMEPANIMALS(GNUM, ANODE, CATHODE, CURRENT)
% gnum: group number as an integer
% anode: the stimulating anode as a string of format 'C#' or 'Rf'
% cathode: the stimulating cathode as a string of format 'C#' or 'Rf'
% current: the stimulating current used for the experimemt as an integer
% SD flag: a boolean flag used to toggle the plotting of SD.

close all;
if ~isempty(varargin) && varargin{1}
    sd_flag = true;
else
    sd_flag = false;
end

% Load common variables
config();
load('config.mat');
load('baseline_struct.mat', 'baseline_struct');

% Selected directories
sel_dirs = {'.\Data\B_animals_sMEP_files\Intact', ...
            '.\Data\B_animals_sMEP_files\W01', ...
            '.\Data\B_animals_sMEP_files\W06'};

% Specific group for analysis
gnum_str = num2str(gnum);
gname = ['Group' gnum_str];

pulse_thold = SLOPE;
nelec_ch = 5;

% Max number of meps for plotting
nmep_lim = 75;

% files of interest
ndirs = length(sel_dirs);

MEP_time = (-PRE3_MEP/fs:1/fs:POST3_MEP/fs)*1000';

% Strings for plot titles
week_temp = split(sel_dirs,'\');
week_names = week_temp(:,:,end);
if gnum == 3
    plt_week = {week_names{1} week_names{2} ...
        [week_names{2} ' BR'] week_names{3}};
else
    plt_week = week_names;
end

foi_inds = length(plt_week);
foi_ind = 1;
ch_names = {'Deltoid', 'Bicep', 'Pronator', 'Flexor', 'Extensor'};

foi = cell(1,1);
grey = ones(3,1).*0.5;

% Loop through selected directories to get all the relevant files of interest
% which match the given configuration
for nsel = 1 : ndirs
    sel_dir = cell2mat(sel_dirs(nsel));
    addpath(genpath(sel_dir));

    g_dir = [sel_dir '\' gname '\'];

    files = dir([g_dir '\*.bin']);
    fnames = {files.name};
    fnames = cellfun(@(x) horzcat(g_dir, x), fnames, 'UniformOutput', false);
    fileinfo_arr = catFilenames(fnames');

    % Match the configuration within the selected directory/group
    anode_arr = catStructField(fileinfo_arr, 'Anode');
    cathode_arr = catStructField(fileinfo_arr, 'Cathode');
    current_arr = catStructField(fileinfo_arr, 'Current');
    current_arr = cell2mat(current_arr);

    arr_inds = 1 : length(anode_arr);

    anode_mask = strcmp(anode_arr, anode);
    cathode_mask = strcmp(cathode_arr, cathode);
    curr_mask = current_arr == current;

    inds = arr_inds(anode_mask & cathode_mask & curr_mask);
    temp = fnames(inds);
    br_mask = contains(temp, 'BR');
    br_inds = inds(br_mask);
    nr_inds = inds(~br_mask);
    foi{:,1} = fnames(nr_inds)';
    foi{:,2} = fnames(br_inds)';

    for c = 1 : size(foi,2)
        foi_len = length(foi{:,c});
        ffoi = foi{c};
        for n = 1 : foi_len
            chosenfile = ffoi{n};
            % Load emg data
            EMG_header = load_emg_header(chosenfile, 1);
            data_array = load_emg_data(chosenfile, ...
                EMG_header,EMG_header.data_position,1,inf,1);
            inds = 1 : size(data_array,1);
            emg_data = data_array(:,1:nelec_ch);
            stim_pulse = data_array(:,6);

            % Filter and normalize against the given configuration baseline
            emg_filt = bpFilter(emg_data, 2, 30, 1e3, nq);
            [~, emg_tkeo] = energyop(emg_filt, false);
            emg_tkeo = reshape(emg_tkeo,size(emg_filt));
            emg_env = abs(lpFilterEMG(emg_tkeo, 2, 50, nq));
            if contains(chosenfile, 'Intact')
                emg_ac = emg_filt;
            else
                heading = chosenfile(1:end-4);
                temp = find(heading(2:end) == 'W') + 1;
                an = heading(temp+4:temp+6);
                emg_norm = normalizeEMGData(emg_filt, ...
                    baseline_struct, an, anode, cathode, current);
                emg_ac = emg_norm;
            end

            % Find threshold
            thold = thresholding(emg_env, THR_MUL);
            active_mask = emg_env > thold;
            new_stim = map_stimpulse(active_mask, stim_pulse);
            for ch = 1 : nelec_ch - 1
                if ch == 1
                    final_stim = and(round(new_stim(:,ch)), ...
                        round(new_stim(:,c+1)));
                else
                    final_stim = and(final_stim, ...
                        round(new_stim(:,ch+1)) );
                end
            end

            % Set pulse offset and get mep traces
            pulse_offset = [final_stim(2:end); NaN];
            risingEdge = inds(final_stim < pulse_thold & ...
                pulse_offset > pulse_thold);

            counter = risingEdge';
            meps = meptrace(emg_ac, counter);
            out_MEPs = meps.out_MEPs;

            % Limit number of traces to nmep_lim variable
            if size(out_MEPs,2) > nmep_lim
                plot_MEPs = out_MEPs(:, 1:nmep_lim, :);
            else
                plot_MEPs = out_MEPs;
            end

            if ~exist('emps_track','var')
                meps_track = plot_MEPs;
            else
                meps_track = horzcat(meps_track, plot_MEPs);
            end
        end
        if exist('meps_track', 'var')
            mu = mean(meps_track, 2);
            sd = std( meps_track, [], 2);
            sd_err = sd./sqrt(size(meps_track,2));
            if c == 1
                treat = 'ND';
            elseif c == 2
                treat = 'BR';
            end
            mep_struct.(treat).(week_names{nsel}).mu = mu;
            mep_struct.(treat).(week_names{nsel}).sd = sd;
            mep_struct.(treat).(week_names{nsel}).sd_err = sd_err;
            clear meps_track;
        end
        % Plot on 1 figure
        if ~isempty(foi{c})
            buff = foi_ind - 2;
            sp_off = 1;
            mu = squeeze(mu);
            sd = squeeze(sd_err);
            for ch = 1 : nelec_ch
                % subplot(nelec_ch, 1, ch);
                subplot(nelec_ch, length(plt_week), ...
                    sp_off + ch + buff);
                hold on;
                if sd_flag
                    plotShadedError(MEP_time, mu(:,ch), sd(:,ch), 'k');
                    % area( MEP_time, mu(:,ch) + sd(:,ch), 'FaceColor', grey,...
                    % 'EdgeColor', grey); 
                    % area( MEP_time, mu(:,ch) - sd(:,ch), 'FaceColor', grey,...
                    % 'EdgeColor', grey); 
                    % plot(MEP_time, mu(:,ch) + sd(:,ch), 'color',      [0.4 0.4 0.4]);
                    % plot(MEP_time, mu(:,ch) - sd(:,ch), 'color', [0.4 0.4 0.4]);
                else
                    plot(MEP_time, mu(:,ch), 'k');
                end
                plot ([0,0], get(gca,'YLim'), 'r');
                plot ([10,10], get(gca,'YLim'), 'b');
                hold off
                xlim([-PRE_MEP POST_MEP]); 
                if ch == 1
                    % title_str = {cell2mat(week_names(n)), cell2mat(ch_names(c))};
                    title_str = plt_week{foi_ind};
                    title(title_str, 'interpreter', 'none', 'fontsize',14);
                end
                if n == 1
                    ylabel(cell2mat(ch_names(c)), 'fontsize', 16);
                end

                if foi_inds == 3
                    sp_off = sp_off + 2;
                elseif foi_inds == 4
                    sp_off = sp_off + 3;
                end
            end
            foi_ind = foi_ind + 1;
        end
    end
end


end

function f_arr = catFilenames(files)
% Concatenate all the files and organise by configurable experiment variables
    f_arr = struct();
    for f = 1 : length(files)
        fileinfo = getFileInfo(files{f});
        Week = fileinfo.Week;
        AnNum = fileinfo.AnNum;
        Anode = fileinfo.Anode;
        Cathode = fileinfo.Cathode;
        Current = fileinfo.Current;

        f_arr(f).Week = Week;
        f_arr(f).AnNum = AnNum;
        f_arr(f).Anode = Anode;
        f_arr(f).Cathode = Cathode;
        f_arr(f).Current = Current;
    end
end

