function plotMEPWeeks(gnum, an, anode, cathode, current)
% This function plots the MEP for a given configuration across the weeks.
% USAGE
% PLOTMEPWEEKS(GNUM, AN, ANODE, CATHODE, CURRENT)
% gnum: group number as an integer
% an: animal number as a string of format 'B##'
% anode: the stimulating anode as a string of format 'C#' or 'Rf'
% cathode: the stimulating cathode as a string of format 'C#' or 'Rf'
% current: the stimulating current used for the experimemt as an integer

% Load common variables
config();
load('config.mat');
load('baseline_struct.mat', 'baseline_struct');

% Selected directories
sel_dirs = {'.\Data\B_animals_sMEP_files\Intact', ...
            '.\Data\B_animals_sMEP_files\W01', ...
            '.\Data\B_animals_sMEP_files\W06'};

% Specific group for analysis
gnum_str = num2str(gnum);
gname = ['Group' gnum_str];

pulse_thold = SLOPE;
nelec_ch = 5;

% Max number of meps for plotting
nmep_lim = 75;

% files of interest
ndirs = length(sel_dirs);
foi = cell(ndirs, 1);

MEP_time = (-PRE3_MEP/fs:1/fs:POST3_MEP/fs)*1000';

% Strings for plot titles
week_temp = split(sel_dirs,'\');
week_names = week_temp(:,:,end);
ch_names = {'Deltoid', 'Bicep', 'Pronator', 'Flexor', 'Extensor'};

foi_ind = 0;

% Loop through selected directories to get all the relevant files of interest
% which match the given configuration
for nsel = 1 : ndirs
    foi_ind = foi_ind + 1;
    sel_dir = cell2mat(sel_dirs(nsel));
    addpath(genpath(sel_dir));

    g_dir = [sel_dir '\' gname];

    files = dir([g_dir '\*.bin']);
    fnames = {files.name};
    fileinfo_arr = catFilenames(fnames');

    % Match the configuration within the selected directory/group
    an_arr = catStructField(fileinfo_arr, 'AnNum');
    anode_arr = catStructField(fileinfo_arr, 'Anode');
    cathode_arr = catStructField(fileinfo_arr, 'Cathode');
    current_arr = catStructField(fileinfo_arr, 'Current');
    current_arr = cell2mat(current_arr);

    arr_inds = 1 : length(an_arr);

    an_mask = strcmp(an_arr, an);
    anode_mask = strcmp(anode_arr, anode);
    cathode_mask = strcmp(cathode_arr, cathode);
    curr_mask = current_arr == current;

    % If we are looking at group 3 and week1, we know there should be 2
    % recording sessions, so repeat the search process otherwise, only take the
    % first one.
    if gnum == 3 && nsel == 2
        inds = arr_inds( an_mask & anode_mask & cathode_mask & curr_mask );
        if length(inds) > 2
            warning('More than 2 files found that match description');
        end
        foi{foi_ind} = [g_dir '\' fnames{inds(1)}];
        foi_ind = foi_ind + 1;
        foi{foi_ind} = [g_dir '\' fnames{inds(2)}];
    else
        [~, ind] = max( an_mask & anode_mask & cathode_mask & curr_mask );
        foi{foi_ind} = [g_dir '\' fnames{ind}];
    end
end

% If it is group 3 then adjust the week name strings for plotting purposes
if foi_ind > 3
    inds = 1 : length(foi);
    br_mask = contains(foi, 'BR');
    br_inds = inds(br_mask);
    if br_inds(1) == 2
        week_names = {week_names{1} [week_names{2} ' BR'] ...
            week_names{2} week_names{3}};
    elseif br_inds(1) == 3
        week_names = {week_names{1} week_names{2} ...
            [week_names{2} ' BR'] week_names{3}};
    end
end

% For each file of interest, perform the given actions
for n = 1 : foi_ind
    chosenfile = foi{n};
    % Load emg data
    EMG_header = load_emg_header(chosenfile, 1);
    data_array = load_emg_data(chosenfile, ...
        EMG_header,EMG_header.data_position,1,inf,1);
    inds = 1 : size(data_array,1);
    emg_data = data_array(:,1:nelec_ch);
    stim_pulse = data_array(:,6);

    % Filter and normalize against the given configuration baseline
    emg_filt = bpFilter(emg_data, 2, 30, 1e3, nq);
    [~, emg_tkeo] = energyop(emg_filt, false);
    emg_tkeo = reshape(emg_tkeo,size(emg_filt));
    emg_env = abs(lpFilterEMG(emg_tkeo, 2, 50, nq));
    if n > 1
        emg_norm = normalizeEMGData(emg_filt, ...
            baseline_struct, an, anode, cathode, current);
        emg_ac = emg_norm;
    else
        emg_ac = emg_filt;
    end

    % Find threshold
    thold = thresholding(emg_env, THR_MUL);
    active_mask = emg_env > thold;
    new_stim = map_stimpulse(active_mask, stim_pulse);
    for c = 1 : nelec_ch - 1
        if c == 1
            final_stim = and(round(new_stim(:,c)), ...
                round(new_stim(:,c+1)));
        else
            final_stim = and(final_stim, ...
                round(new_stim(:,c+1)) );
        end
    end

    % Set pulse offset and get mep traces
    pulse_offset = [final_stim(2:end); NaN];
    risingEdge = inds(final_stim < pulse_thold & ...
        pulse_offset > pulse_thold);

    counter = risingEdge';
    meps = meptrace(emg_ac, counter);
    out_MEPs = meps.out_MEPs;

    % Limit number of traces to nmep_lim variable
    if size(out_MEPs,2) > nmep_lim
        plot_MEPs = out_MEPs(:, 1:nmep_lim, :);
    else
        plot_MEPs = out_MEPs;
    end
    offset = repmat(1:size(plot_MEPs,2), size(plot_MEPs,1), 1);

    % Plot on 1 figure
    buff = n - 2;
    sp_off = 1;
    for c = 1 : nelec_ch
        subplot(nelec_ch, foi_ind, ...
            sp_off + c + buff);
        plot(MEP_time, plot_MEPs(:, :, c)+offset, 'k');
        hold on
        plot ([0,0], [19,61], 'r');
        plot ([10,10], [19,61], 'b');
        hold off
        ylim([19, 61]);
        xlim([-PRE_MEP POST_MEP]); 
        if c == 1
            % title_str = {cell2mat(week_names(n)), cell2mat(ch_names(c))};
            title_str = cell2mat(week_names(n));
            title(title_str, 'interpreter', 'none', 'fontsize',14);
        end
        if n == 1
            ylabel(cell2mat(ch_names(c)), 'fontsize', 16);
        end

        if foi_ind == 3
            sp_off = sp_off + 2;
        elseif foi_ind == 4
            sp_off = sp_off + 3;
        end
    end

end

end

function f_arr = catFilenames(files)
% Concatenate all the files and organise by configurable experiment variables
    f_arr = struct();
    for f = 1 : length(files)
        fileinfo = getFileInfo(files{f});
        Week = fileinfo.Week;
        AnNum = fileinfo.AnNum;
        Anode = fileinfo.Anode;
        Cathode = fileinfo.Cathode;
        Current = fileinfo.Current;

        f_arr(f).Week = Week;
        f_arr(f).AnNum = AnNum;
        f_arr(f).Anode = Anode;
        f_arr(f).Cathode = Cathode;
        f_arr(f).Current = Current;
    end
end

